@@ -1,5 +1,3 @@
-from __future__ import division, print_function
-
 import os.path
 import pickle
 
@@ -186,6 +184,9 @@ class TestCurveFitter(object):
         chain = mcfitter.chain
         mcfitter.initialise(pos=chain)
         assert_equal(mcfitter._state.coords, chain[-1])
+        # initialise with chain if it's never been run before
+        mcfitter = CurveFitter(self.objective, nwalkers=100)
+        mcfitter.initialise(chain)
 
         if not _HAVE_PTSAMPLER:
             return
@@ -207,6 +208,9 @@ class TestCurveFitter(object):
         chain = mcfitter.chain
         mcfitter.initialise(pos=np.copy(chain))
         assert_equal(mcfitter._state.coords, chain[-1])
+        # initialise with chain if it's never been run before
+        mcfitter = CurveFitter(self.objective, nwalkers=100, ntemps=20)
+        mcfitter.initialise(chain)
 
     def test_fit_smoke(self):
         # smoke tests to check that fit runs
@@ -350,6 +354,24 @@ class TestFitterGauss(object):
         f.sample(steps=2, nthin=4, f=checkpoint, verbose=False)
         assert_equal(f.chain.shape[0], 2)
 
+        # we should be able to produce 2 * 100 steps from the generator
+        g = self.objective.pgen(ngen=20000000000)
+        s = [i for i, a in enumerate(g)]
+        assert_equal(np.max(s), 200 - 1)
+        g = self.objective.pgen(ngen=200)
+        pvec = next(g)
+        assert_equal(pvec.size, len(self.objective.parameters.flattened()))
+
+        # check that all the parameters are returned via pgen, not only those
+        # being varied.
+        self.params[0].vary = False
+        f = CurveFitter(self.objective, nwalkers=100)
+        f.initialise('jitter')
+        f.sample(steps=2, nthin=4, f=checkpoint, verbose=False)
+        g = self.objective.pgen(ngen=100)
+        pvec = next(g)
+        assert_equal(pvec.size, len(self.objective.parameters.flattened()))
+
         # the following test won't work because of emcee/gh226.
         # chain = load_chain(checkpoint)
         # assert_(chain.shape == f.chain.shape)
@@ -379,6 +401,16 @@ class TestFitterGauss(object):
         # assert_allclose(uncertainties, self.best_unweighted_errors,
         #                 rtol=0.15)
 
+    def test_all_minimisers(self):
+        """test minimisers against the Gaussian fit"""
+        f = CurveFitter(self.objective)
+
+        for method in ['differential_evolution', 'shgo', 'dual_annealing',
+                       'L-BFGS-B', 'least_squares']:
+            self.objective.setp(self.p0)
+            res = f.fit(method=method)
+            assert_almost_equal(res.x, self.best_weighted, 3)
+
 
 """
         The Gaussian example sampling can also be performed with pymc3.
