@@ -2,7 +2,6 @@
 Test co-refinement of datasets by fitting 3 neutron reflectivity datasets. The
 overall construction of the models can be done in a few different ways.
 """
-from __future__ import print_function, division
 import os.path
 
 import numpy as np
@@ -131,9 +130,9 @@ class TestGlobalFitting(object):
         assert_almost_equal(global_chisqr, indiv_chisqr)
 
         # now check that the parameters were held in common correctly.
-        slabs361 = structure361.slabs
-        slabs365 = structure365.slabs
-        slabs366 = structure366.slabs
+        slabs361 = structure361.slabs()
+        slabs365 = structure365.slabs()
+        slabs366 = structure366.slabs()
 
         assert_equal(slabs365[0:2, 0:5], slabs361[0:2, 0:5])
         assert_equal(slabs366[0:2, 0:5], slabs361[0:2, 0:5])
